@@ -9,24 +9,24 @@ import (
 	"github.com/spf13/cobra"
 )
 
-const longCompletion = `Generate shell autocompletions. Valid arguments are bash, zsh, and fish.`
+const longCompletion = `Generate shell autocompletions. Valid arguments are bash, fish, zsh, and powershell.`
 
 var (
 	// Todo --noDesc param?
 	noDesc = false
 
-	shells = []string{"bash", "zsh", "fish", "powershell"}
+	shells = []string{"bash", "fish", "zsh", "powershell"}
 
 	// completionCmd creates completion shell files
 	completionCmd = &cobra.Command{
 		Use:                   fmt.Sprintf("completion [%s]", strings.Join(shells, "|")),
-		Short:                 "Generate shell autocompletions",
+		Short:                 "Generate shell autocompletions.",
 		Long:                  longCompletion,
 		DisableSuggestions:    false,
 		DisableFlagsInUseLine: true,
 		ValidArgs:             shells,
 		Run:                   completion,
-		Hidden:                true,
+		Hidden:                false,
 	}
 )
 
@@ -38,14 +38,14 @@ func completion(cmd *cobra.Command, args []string) {
 	switch args[0] {
 	case "bash":
 		cmd.Root().GenBashCompletionV2(os.Stdout, !noDesc)
+	case "fish":
+		cmd.Root().GenFishCompletion(os.Stdout, !noDesc)
 	case "zsh":
 		if noDesc {
 			cmd.Root().GenZshCompletionNoDesc(os.Stdout)
 		} else {
 			cmd.Root().GenZshCompletion(os.Stdout)
 		}
-	case "fish":
-		cmd.Root().GenFishCompletion(os.Stdout, !noDesc)
 	case "powershell":
 		if noDesc {
 			cmd.Root().GenPowerShellCompletion(os.Stdout)
