@@ -12,7 +12,7 @@ import (
 	"github.com/beringresearch/macpine/host"
 	"github.com/beringresearch/macpine/qemu"
 	"github.com/spf13/cobra"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 )
 
 // listCmd lists Alpine instances
@@ -33,13 +33,9 @@ func list(cmd *cobra.Command, args []string) {
 
 	status := []string{}
 	config := []qemu.MachineConfig{}
-	pid := []int{}
-
-	vmNames, err := host.ListVMNames()
-	if err != nil {
-		log.Fatal(err)
-	}
+	pid := []string{}
 
+	vmNames := host.ListVMNames()
 	for _, f := range vmNames {
 
 		machineConfig := qemu.MachineConfig{}
@@ -57,13 +53,28 @@ func list(cmd *cobra.Command, args []string) {
 
 		s, p := host.Status(machineConfig)
 		status = append(status, s)
-		pid = append(pid, p)
+		if s == "Stopped" {
+			pid = append(pid, "-")
+		} else {
+			pid = append(pid, fmt.Sprint(p))
+		}
 	}
 
 	w := tabwriter.NewWriter(os.Stdout, 1, 1, 1, ' ', 0)
-	fmt.Fprintln(w, "NAME\tOS\tSTATUS\tSSH\tPORTS\tARCH\tPID\t")
+	fmt.Fprintln(w, "NAME\tOS\tSTATUS\tSSH\tPORTS\tARCH\tPID\tTAGS\t")
 	for i, machine := range config {
-		fmt.Fprintln(w, machine.Alias+"    \t"+strings.Split(machine.Image, "_")[0]+"    \t"+status[i]+"    \t"+machine.SSHPort+"    \t"+machine.Port+"    \t"+machine.Arch+"    \t"+fmt.Sprint(pid[i])+"    \t")
+		spacer := "    \t"
+		row := []string{
+			machine.Alias,
+			strings.Split(machine.Image, "_")[0],
+			status[i],
+			machine.SSHPort,
+			machine.Port,
+			machine.Arch,
+			fmt.Sprint(pid[i]),
+			strings.Join(machine.Tags, ","),
+		}
+		fmt.Fprintln(w, strings.Join(row, "    \t")+spacer)
 	}
 	w.Flush()
 
