@@ -11,7 +11,7 @@ import (
 	qemu "github.com/beringresearch/macpine/qemu"
 	"github.com/beringresearch/macpine/utils"
 	"github.com/spf13/cobra"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 )
 
 // publishCmd stops an Alpine instance
@@ -35,10 +35,7 @@ func publish(cmd *cobra.Command, args []string) {
 		log.Fatal("missing VM name")
 	}
 
-	vmList, err := host.ListVMNames()
-	if err != nil {
-		log.Fatal(err)
-	}
+	vmList := host.ListVMNames()
 
 	exists := utils.StringSliceContains(vmList, args[0])
 	if !exists {
@@ -88,6 +85,7 @@ func publish(cmd *cobra.Command, args []string) {
 
 	err = host.Start(machineConfig)
 	if err != nil {
+		host.Stop(machineConfig)
 		log.Fatal(err)
 	}
 
