@@ -10,7 +10,7 @@ import (
 	qemu "github.com/beringresearch/macpine/qemu"
 	"github.com/beringresearch/macpine/utils"
 	"github.com/spf13/cobra"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 )
 
 // startCmd starts an Alpine instance
@@ -34,11 +34,7 @@ func start(cmd *cobra.Command, args []string) {
 		log.Fatal("missing VM name")
 	}
 
-	vmList, err := host.ListVMNames()
-	if err != nil {
-		log.Fatal(err)
-	}
-
+	vmList := host.ListVMNames()
 	exists := utils.StringSliceContains(vmList, args[0])
 	if !exists {
 		log.Fatal("unknown machine " + args[0])
@@ -58,7 +54,7 @@ func start(cmd *cobra.Command, args []string) {
 
 	err = host.Start(machineConfig)
 	if err != nil {
+		host.Stop(machineConfig)
 		log.Fatal(err)
 	}
-
 }
