@@ -2,42 +2,35 @@ package host
 
 import (
 	"io/ioutil"
-	"log"
 	"os"
 	"path/filepath"
 
 	"github.com/spf13/cobra"
 )
 
-func ListVMNames() ([]string, error) {
+func ListVMNames() []string {
 	var vmList []string
 
 	userHomeDir, err := os.UserHomeDir()
 	if err != nil {
-		return vmList, nil
+		return vmList
 	}
 
 	dirList, err := ioutil.ReadDir(filepath.Join(userHomeDir, ".macpine"))
 	if err != nil {
-		return vmList, nil
+		return vmList
 	}
 
 	for _, f := range dirList {
 		if f.Name() != "cache" {
 			vmList = append(vmList, f.Name())
 		}
-
 	}
 
-	return vmList, nil
+	return vmList
 }
 
 // autocomplete with VM Names
 func AutoCompleteVMNames(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-	listVMNames, err := ListVMNames()
-	if err != nil {
-		log.Fatal(err)
-	}
-
-	return listVMNames, cobra.ShellCompDirectiveNoFileComp
+	return ListVMNames(), cobra.ShellCompDirectiveNoFileComp
 }
