@@ -5,6 +5,7 @@ import (
 	"fmt"
 	"io/ioutil"
 	"log"
+	"net"
 	"os"
 	"os/exec"
 	"path/filepath"
@@ -12,45 +13,77 @@ import (
 	"strconv"
 	"strings"
 	"syscall"
-	"time"
 
 	"github.com/beringresearch/macpine/utils"
 	"golang.org/x/crypto/ssh"
+	"golang.org/x/crypto/ssh/agent"
 	"golang.org/x/term"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 )
 
 type MachineConfig struct {
-	Alias       string `yaml:"alias"`
-	Image       string `yaml:"image"`
-	Arch        string `yaml:"arch"`
-	CPU         string `yaml:"cpu"`
-	Memory      string `yaml:"memory"`
-	Disk        string `yaml:"disk"`
-	Mount       string `yaml:"mount"`
-	Port        string `yaml:"port"`
-	SSHPort     string `yaml:"sshport"`
-	SSHUser     string `yaml:"sshuser"`
-	SSHPassword string `yaml:"sshpassword"`
-	MACAddress  string `yaml:"macaddress"`
-	Location    string `yaml:"location"`
+	Alias        string   `yaml:"alias"`
+	Image        string   `yaml:"image"`
+	Arch         string   `yaml:"arch"`
+	CPU          string   `yaml:"cpu"`
+	Memory       string   `yaml:"memory"`
+	Disk         string   `yaml:"disk"`
+	Mount        string   `yaml:"mount"`
+	Port         string   `yaml:"port"`
+	SSHPort      string   `yaml:"sshport"`
+	SSHUser      string   `yaml:"sshuser"`
+	SSHPassword  string   `yaml:"sshpassword"`
+	RootPassword *string  `yaml:"rootpassword,omitempty"`
+	MACAddress   string   `yaml:"macaddress"`
+	Location     string   `yaml:"location"`
+	Tags         []string `yaml:"tags"`
 }
 
 // Exec starts an interactive shell terminal in VM
-func (c *MachineConfig) Exec(cmd string) error {
-
+func (c *MachineConfig) Exec(cmd string, root bool) error {
+	if cmd == "" {
+		return nil
+	}
 	host := "localhost:" + c.SSHPort
 	user := c.SSHUser
 	pwd := c.SSHPassword
+	if root && user != "root" {
+		user = "root"
+		if c.RootPassword == nil {
+			pwd = "root"
+		} else {
+			pwd = *c.RootPassword
+		}
+	}
+	cred, err := utils.GetCredential(pwd)
+	if err != nil {
+		return err
+	}
 
-	var err error
-
-	conf := &ssh.ClientConfig{
-		User:            user,
-		HostKeyCallback: ssh.InsecureIgnoreHostKey(),
-		Auth: []ssh.AuthMethod{
-			ssh.Password(pwd),
-		},
+	var conf *ssh.ClientConfig
+	if cred.CRType == utils.PwdCred {
+		conf = &ssh.ClientConfig{
+			User:            user,
+			HostKeyCallback: ssh.InsecureIgnoreHostKey(),
+			Auth: []ssh.AuthMethod{
+				ssh.Password(cred.CR),
+			},
+		}
+	} else { // utils.HostCred
+		// Use SSH agent (https://pkg.go.dev/golang.org/x/crypto/ssh/agent#example-NewClient)
+		socket := os.Getenv("SSH_AUTH_SOCK")
+		conn, err := net.Dial("unix", socket)
+		if err != nil {
+			log.Fatalf("failed to open SSH_AUTH_SOCK: %v", err)
+		}
+		agentClient := agent.NewClient(conn)
+		conf = &ssh.ClientConfig{
+			User: user,
+			Auth: []ssh.AuthMethod{
+				ssh.PublicKeysCallback(agentClient.Signers),
+			},
+			HostKeyCallback: ssh.InsecureIgnoreHostKey(),
+		}
 	}
 
 	var conn *ssh.Client
@@ -67,7 +100,8 @@ func (c *MachineConfig) Exec(cmd string) error {
 	}
 	defer session.Close()
 
-	if (cmd == "ash") || cmd == ("bash") {
+	// XXX get shells from /etc/shells instead?
+	if (cmd == "ash") || (cmd == "bash") {
 		err := attachShell(session)
 		if err != nil {
 			return err
@@ -81,9 +115,7 @@ func (c *MachineConfig) Exec(cmd string) error {
 			return err
 		}
 	}
-
 	return nil
-
 }
 
 func attachShell(session *ssh.Session) error {
@@ -140,7 +172,7 @@ func attachShell(session *ssh.Session) error {
 
 // Status returns VM status
 func (c *MachineConfig) Status() (string, int) {
-	status := ""
+	status := "Stopped"
 	var pid int
 
 	pidFile := filepath.Join(c.Location, "alpine.pid")
@@ -156,40 +188,31 @@ func (c *MachineConfig) Status() (string, int) {
 		process := string(vmPID)
 		process = strings.TrimSuffix(process, "\n")
 		pid, _ = strconv.Atoi(process)
-	} else {
-		status = "Stopped"
 	}
 	return status, pid
 }
 
 // Stop stops an Alpine VM
 func (c *MachineConfig) Stop() error {
-	pidFile := filepath.Join(c.Location, "alpine.pid")
-	if _, err := os.Stat(pidFile); err == nil {
-
-		_, pid := c.Status()
-
+	// qemu creates PID file with -pidfile flag, and deletes it on sigterm
+	if status, pid := c.Status(); status != "Stopped" {
 		if pid > 0 {
-
-			// err = syscall.Kill(pid, 15)
-			// if err != nil {
-			// 	return err
-			// }
-
 			p, procErr := os.FindProcess(pid)
-
 			if procErr != nil {
 				return procErr
 			}
-
-			return p.Signal(syscall.SIGTERM)
-
+			if err := p.Signal(syscall.SIGTERM); err != nil {
+				return err
+			}
+			log.Println(c.Alias + " stopped")
+			return nil
 		} else {
-			return errors.New("unable to SIGTERM 15: incorrect PID")
+			pidFile := filepath.Join(c.Location, "alpine.pid")
+			return errors.New("error stopping, incorrect PID in " + pidFile + "?")
 		}
+	} else {
+		return errors.New(c.Alias + " is not running")
 	}
-
-	return nil
 }
 
 func getHostArchitecture() (string, error) {
@@ -230,6 +253,7 @@ func (c *MachineConfig) GetAccel() string {
 			return "whpx" // untested
 		}
 	}
+	log.Println("Note: defaulting to QEMU tiny codegen. Emulation overhead may be significant.")
 	return "tcg,tb-size=1024,thread=multi"
 }
 
@@ -238,10 +262,17 @@ func (c *MachineConfig) Start() error {
 
 	exposedPorts := "user,id=net0,hostfwd=tcp::" + c.SSHPort + "-:22"
 
-	if c.Port != "" {
-		s := strings.Split(c.Port, ",")
-		for _, p := range s {
-			exposedPorts += ",hostfwd=tcp::" + p + "-:" + p
+	ports, err := utils.ParsePort(c.Port)
+	if err != nil {
+		log.Fatalf("Error configuring ports: %v\n", err)
+	}
+	for _, p := range ports {
+		hostp := strconv.Itoa(p.Host)
+		guestp := strconv.Itoa(p.Guest)
+		if p.Proto == utils.Tcp {
+			exposedPorts += ",hostfwd=tcp::" + hostp + "-:" + guestp
+		} else { // Udp
+			exposedPorts += ",hostfwd=udp::" + hostp + "-:" + guestp
 		}
 	}
 
@@ -252,7 +283,7 @@ func (c *MachineConfig) Start() error {
 	cpuType := map[string]string{
 		"aarch64":      "cortex-a72",
 		"x86_64":       "qemu64",
-		"x86_64_host":  "host,-pdpe1gb",
+		"x86_64_host":  "host,-pdpe1gb=on",
 		"aarch64_host": "host"}
 
 	hostCPU := ""
@@ -303,6 +334,7 @@ func (c *MachineConfig) Start() error {
 		"-parallel", "none",
 		"-device", "virtio-rng-pci",
 		"-rtc", "base=utc,clock=host",
+		"-daemonize",
 		"-name", c.Alias}
 
 	if c.Arch == "aarch64" {
@@ -323,45 +355,41 @@ func (c *MachineConfig) Start() error {
 	// Uncomment to debug qemu messages
 	cmd.Stderr = os.Stderr
 
-	log.Printf("Booting...")
-	err = cmd.Start()
+	log.Println("booting " + c.Alias)
+	err = cmd.Run()
 	if err != nil {
+		c.Stop()
+		c.CleanPIDFile()
 		return err
 	}
 
-	err = utils.Retry(10, 2*time.Second, func() error {
-		err := c.Exec("hwclock -s")
-		if err != nil {
-			return err
-		}
-		return nil
-	})
+	log.Println("awaiting ssh server...")
+	err = c.Exec("hwclock -w", true) // root=true i.e. run as root
 	if err != nil {
-		return errors.New("unable to sync clocks: " + err.Error())
+		c.Stop()
+		c.CleanPIDFile()
+		return err
 	}
 
 	if c.Mount != "" {
-		err = utils.Retry(10, 2*time.Second, func() error {
-			err := c.Exec("DIR=$(eval echo \"~$USER\");mkdir -p $DIR/mnt/; chmod 777 $DIR/mnt; mount -t 9p -o trans=virtio host0 $DIR/mnt/ -oversion=9p2000.L,msize=104857600")
-			if err != nil {
-				return err
-			}
-			return nil
-		})
-
-		if err != nil {
-			return errors.New("unable to mount: " + err.Error())
+		basename := filepath.Base(c.Mount)
+		mntcmd := make([]string, 3)
+		mntcmd[0] = "mkdir -p /mnt/" + basename
+		mntcmd[1] = "chmod 777 /mnt/" + basename
+		mntcmd[2] = "mount -t 9p -o trans=virtio,version=9p2000.L,msize=104857600 host0 /mnt/" + basename
+		if err := c.Exec(strings.Join(mntcmd, " && "), true); err != nil {
+			log.Println("error mounting directory: " + err.Error())
+		} else {
+			log.Println("mounted " + c.Mount + " on /mnt/" + basename)
 		}
-
-		log.Printf("Mounted " + c.Mount + " --> $HOME/mnt/")
 	}
 
-	status, _ := c.Status()
-
+	status, pid := c.Status()
 	if status == "Stopped" {
 		return errors.New("unable to start vm")
 	}
 
+	log.Println(c.Alias + " started (" + strconv.Itoa(pid) + ")")
 	return nil
 }
 
@@ -442,46 +470,29 @@ func (c *MachineConfig) Launch() error {
 		return errors.New("unable launch a new machine. " + err.Error())
 	}
 
-	//Resize disk on an alpine guest
+	// Resize disk on an alpine guest
 	if strings.Split(c.Image, "_")[0] == "alpine" {
-		err = utils.Retry(10, 1*time.Second, func() error {
-			err := c.Exec("apk add --no-cache e2fsprogs-extra sfdisk partx")
-			if err != nil {
-				return err
-			}
-
-			return nil
-		})
+		//TODO add these dependencies into pre-baked macpine image
+		err := c.Exec("apk add --no-cache e2fsprogs-extra sfdisk partx", true) // root=true i.e. run as root
 		if err != nil {
-			return errors.New("unable install dependencies: " + err.Error())
+			return errors.New("unable to install dependencies: " + err.Error())
 		}
 
-		err = utils.Retry(3, 1*time.Second, func() error {
-			err = c.Exec(`echo ", +" | sfdisk --no-reread -N 3 /dev/vda; partx -u /dev/vda`)
-			if err != nil {
-				return err
-			}
-
-			return nil
-		})
+		// send sfdisk command ,+ (<start>,<size>,<type>,<bootable>)
+		// default start (0), size + (all available), default type (linux data), default bootable (false)
+		err = c.Exec(`echo ",+" | sfdisk --no-reread --partno 3 /dev/vda && partx -u /dev/vda`, true)
 		if err != nil {
-			return errors.New("error expanding disk: " + err.Error())
+			return errors.New("error updating partition table: " + err.Error())
 		}
 
-		err = utils.Retry(3, 1*time.Second, func() error {
-			err = c.Exec("resize2fs /dev/vda3")
-			if err != nil {
-				return err
-			}
-
-			return nil
-		})
+		err = c.Exec("resize2fs /dev/vda3", true)
 		if err != nil {
-			return errors.New("error expanding disk: " + err.Error())
+			return errors.New("error expanding filesystem: " + err.Error())
 		}
 
-		err = c.Exec("df -h")
+		err = c.Exec("df -h", true)
 		if err != nil {
+
 			return err
 		}
 	}
@@ -529,3 +540,10 @@ func (c *MachineConfig) CreateQemuDiskImage(imageName string) error {
 
 	return nil
 }
+
+func (c *MachineConfig) CleanPIDFile() {
+	pidFile := filepath.Join(c.Location, "alpine.pid")
+	if err := os.Remove(pidFile); err != nil && !errors.Is(err, os.ErrNotExist) {
+		log.Fatalf("error deleting pidfile at %s. Manually delete it before proceeding.", pidFile)
+	}
+}
